-- Generation of timed automata modeling critical region example from PAT Benchmarks
-- built from Uppaal model available at http://www.labri.fr/perso/herbrete/tchecker/
import System.Environment

-- Generation of the kth ProdCell
genProdCell :: Int -> Int -> Int -> String
genProdCell k a b =
   "auto prodCell" ++ (show k) ++ "\n" ++
   "{\n" ++
   "   notready -- private1of" ++ (show k) ++ " | x" ++ (show k) ++ "<=" ++ (show b) ++ " x" ++ (show k) ++ ":=0 -> testing\n" ++
   "   testing -- private2of" ++ (show k) ++ " | x" ++ (show k) ++ ">=" ++ (show a) ++ " x" ++ (show k) ++ ":=0 -> notready\n" ++
   "   testing -- private3of" ++ (show k) ++ " | x" ++ (show k) ++ "<=" ++ (show (a - 1)) ++ " -> requesting\n" ++
   "   requesting -- cellenter" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> critical\n" ++
   "   critical -- cellexit" ++ (show k) ++ " | x" ++ (show k) ++ "<=" ++ (show (a - 1)) ++ " x" ++ (show k) ++ ":=0 -> testing2\n" ++
   "   critical -- private4of" ++ (show k) ++ " | x" ++ (show k) ++ ">=" ++ (show b) ++ " -> error\n" ++
   "   testing2 -- private5of" ++ (show k) ++ " | x" ++ (show k) ++ "<=" ++ (show (a - 1)) ++ " -> safe\n" ++
   "   testing2 -- private6of" ++ (show k) ++ " | x" ++ (show k) ++ ">=" ++ (show a) ++ " -> error\n" ++
   "   [testing x" ++ (show k) ++ "<=" ++ (show a) ++"]\n" ++
   "   [critical x" ++ (show k) ++ "<=" ++ (show b) ++"]\n" ++
   "   [testing2 x" ++ (show k) ++ "<=" ++ (show a) ++"]\n" ++
   "}\n"

-- Generation of the counter
genCounter :: Int -> String
genCounter numProdCell = 
   "auto counter\n" ++
   "{\n" ++
   "   counter0 -- initCounter -> counter1\n" ++ 
   (aux 1) ++ 
   "}\n"
   where
     aux k
        | k >= numProdCell =
           "   counter" ++ (show k) ++ " -- incCounter -> counter1\n" ++
           "   counter" ++ (show k) ++ " -- cellenter" ++ (show k) ++ " -> counter0\n" ++
           (auxAux 1 k)
        | otherwise =
           "   counter" ++ (show k) ++ " -- incCounter -> counter" ++ (show (k + 1)) ++ "\n" ++
           "   counter" ++ (show k) ++ " -- cellenter" ++ (show k) ++ " -> counter0\n" ++
           (auxAux 1 k) ++
           (aux (k + 1))
     auxAux c k
        | c > numProdCell =
           ""
        | otherwise =
           "   counter" ++ (show k) ++ " -- cellexit" ++ (show c) ++ " -> counter" ++ (show c) ++ "\n" ++
           (auxAux (c + 1) k)

-- Generation of the kth Arbiter
genArbiter :: Int -> String
genArbiter k =
  "auto arbiter" ++ (show k) ++ "\n" ++
  "{\n" ++
  "   out -- cellenter" ++ (show k) ++ " -> in\n" ++
  "   in -- cellexit" ++ (show k) ++ " -> out\n" ++
  "}\n"


-- Generation of the objective
genObj :: String
genObj  = "objective prodCell1.error"

-- Generation of the automata for numC cells
genAll :: (String,String) -> String
genAll (sNumC, sK) = auxGen numC ""
   where
      auxGen 0 s = (genCounter numC) ++ s ++ genObj
      auxGen k s = auxGen (k-1) ((genProdCell k 10 20) ++ (genArbiter k) ++ s)
      -- k = read sK
      numC = read sNumC

-- Gets the two first elements of a list
fstTwo :: [a] -> (a,a)
fstTwo l = (head l, head (tail l))

-- Prints the automata generated for a number of cells given as an argument
main = fmap (genAll.fstTwo) getArgs >>= putStr
