-- Generation of timed automata modeling FDDI example
-- built from Uppaal model available at http://www.labri.fr/perso/herbrete/tchecker/
--  property: station1 and station2 must not be in cs together
import System.Environment

-- Generation of the kth Station
genStation :: Int -> Int -> Int -> String
genStation k ttrt sa =
   "auto station" ++ (show k) ++ "\n" ++
   "{\n" ++
   "   q0 -- TT" ++ (show k) ++ " | trt" ++ (show k) ++ ">=" ++ (show ttrt) ++ " trt" ++ (show k) ++ ":=0 xB" ++ (show k) ++ ":=0 -> q1\n" ++
   "   q0 -- TT" ++ (show k) ++ " | trt" ++ (show k) ++ "<" ++ (show ttrt) ++ " trt" ++ (show k) ++ ":=0 xB" ++ (show k) ++ ":=0 -> q2\n" ++
   "   q1 -- RT" ++ (show k) ++ " | trt" ++ (show k) ++ "==" ++ (show sa) ++ " -> q4\n" ++
   "   q2 -- private1of" ++ (show k) ++ " | trt" ++ (show k) ++ "==" ++ (show sa) ++ " -> q3\n" ++
   "   q3 -- RT" ++ (show k) ++ " -> q4\n" ++
   "   q4 -- TT" ++ (show k) ++ " | trt" ++ (show k) ++ ">=" ++ (show ttrt) ++ " trt" ++ (show k) ++ ":=0 xA" ++ (show k) ++ ":=0 -> q5\n" ++
   "   q4 -- TT" ++ (show k) ++ " | trt" ++ (show k) ++ "<" ++ (show ttrt) ++ " trt" ++ (show k) ++ ":=0 xA" ++ (show k) ++ ":=0 -> q6\n" ++
   "   q5 -- RT" ++ (show k) ++ " | trt" ++ (show k) ++ "==" ++ (show sa) ++ " -> q0\n" ++
   "   q6 -- private2of" ++ (show k) ++ " | trt" ++ (show k) ++ "==" ++ (show sa) ++ " -> q7\n" ++
   "   q7 -- RT" ++ (show k) ++ " -> q0\n" ++
   "   [q1 trt" ++ (show k) ++ "<=" ++ (show sa) ++"]\n" ++
   "   [q2 trt" ++ (show k) ++ "<=" ++ (show sa) ++"]\n" ++
   "   [q3 xA" ++ (show k) ++ "<=" ++ (show (ttrt + sa)) ++ "]\n" ++
   "   [q5 trt" ++ (show k) ++ "<=" ++ (show sa) ++ "]\n" ++
   "   [q6 trt" ++ (show k) ++ "<=" ++ (show sa) ++ "]\n" ++
   "   [q7 xB" ++ (show k) ++ "<=" ++ (show (ttrt + sa)) ++ "]\n" ++ 
   "}\n"

-- Generation of the ring
genRing :: Int -> Int -> String
genRing numStation td = 
   "auto ring\n" ++
   "{\n" ++
   (aux 1) ++
   (aux2 1) ++
   "}\n"
   where
      aux k
         | k >= numStation =
            "   sa" ++ (show k) ++ " -- TT" ++ (show k) ++ " | t==" ++ (show td) ++ " -> sb" ++ (show k) ++ "\n" ++
            "   sb" ++ (show k) ++ " -- RT" ++ (show k) ++ " | t:=0 -> sa" ++ (show 1) ++ "\n"
         | otherwise =
            "   sa" ++ (show k) ++ " -- TT" ++ (show k) ++ " | t==" ++ (show td) ++ " -> sb" ++ (show k) ++ "\n" ++
            "   sb" ++ (show k) ++ " -- RT" ++ (show k) ++ " | t:=0 -> sa" ++ (show (k + 1)) ++ "\n" ++
            (aux (k + 1))
      aux2 k
         | k > numStation =
            ""
         | otherwise =
            "   [sa" ++ (show k) ++ " t<=" ++ (show td) ++ "]\n" ++
            aux2 (k + 1)

-- Generation of the objective
genObj :: String
genObj  = "objective station2.q1 & station3.q1"
-- genObj  = "objective station1.q5 & station2.q5"
-- genObj  = "objective station1.q1 & station2.q1"

-- Generation of the automata for numC stations
genAll :: (String,String) -> String
genAll (sNumC, sK) = auxGen numC ""
   where
      auxGen 0 s = (genRing numC 0) ++ s ++ genObj
      auxGen k s = auxGen (k-1) ((genStation k (50 * numC) 20) ++ s)
      -- k = read sK
      numC = read sNumC

-- Gets the two first elements of a list
fstTwo :: [a] -> (a,a)
fstTwo l = (head l, head (tail l))

-- Prints the automata generated for a number of stations given as an argument
main = fmap (genAll.fstTwo) getArgs >>= putStr
