-- Generation of timed automata modeling Fischer's protocol with bad time constraints
--  Processes 1 and 2 should not be in CS together
import System.Environment

-- Generation of the kth process with parameter p
genP :: Int -> Int -> String
genP k p
   | k == 1 =
     "auto P" ++ (show k) ++ "\n" ++
     "{\n" ++
     "   a -- idis0for" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> req\n" ++
     "   req -- setid" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 x" ++ (show k) ++ "<=" ++ (show (p + 1)) ++ " -> wait\n" ++
     "   wait -- idis" ++ (show k) ++ " | x" ++ (show k) ++ ">" ++ (show p) ++ " -> cs\n" ++
     "   wait -- idis0for" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> req\n" ++
     "   cs -- setid0by" ++ (show k) ++ " -> a\n" ++
     "   [req x" ++ (show k) ++ "<=" ++ (show (p + 1)) ++ "]\n" ++
     "}\n"
   | otherwise =
     "auto P" ++ (show k) ++ "\n" ++
     "{\n" ++
     "   a -- idis0for" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> req\n" ++
     "   req -- setid" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 x" ++ (show k) ++ "<=" ++ (show p) ++ " -> wait\n" ++
     "   wait -- idis" ++ (show k) ++ " | x" ++ (show k) ++ ">" ++ (show p) ++ " -> cs\n" ++
     "   wait -- idis0for" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> req\n" ++
     "   cs -- setid0by" ++ (show k) ++ " -> a\n" ++
     "   [req x" ++ (show k) ++ "<=" ++ (show p) ++ "]\n" ++
     "}\n"

-- Generation of shared variable id with numP processes
genId :: Int -> String
genId numP =
   "auto id\n" ++
   "{\n" ++
   (aux1 1) ++ -- idis0for
   (aux2 1) ++ -- idis
   (aux3 0) ++ -- setid0by
   (aux4 0) ++ -- setid
   "}\n"
   where
      aux1 k
         | k > numP = ""
         | otherwise =
            "   id0 -- idis0for" ++ (show k) ++ " -> id0\n" ++
            (aux1 (k + 1))
      aux2 k
         | k > numP = ""
         | otherwise =
            "   id" ++ (show k) ++ " -- idis" ++ (show k) ++ " -> id" ++ (show k) ++ "\n" ++
            (aux2 (k + 1))
      aux3 k
         | k > numP = ""
         | otherwise = auxAux3 1
         where
            auxAux3 l
               | l > numP = aux3 (k + 1)
               | otherwise =
                  "   id" ++ (show k) ++ " -- setid0by" ++ (show l) ++ " -> id0\n" ++
                  (auxAux3 (l + 1))
      aux4 k
         | k > numP = ""
         | otherwise = auxAux4 1
         where
            auxAux4 l
               | l > numP = aux4 (k + 1)
               | otherwise =
                  "   id" ++ (show k) ++ " -- setid" ++ (show l) ++ " -> id" ++ (show l) ++ "\n" ++
                  (auxAux4 (l + 1)) 

-- Generation of the objective
genObj :: String
genObj  = "objective P1.cs & P2.cs"

-- Generation of the automata for numP processes
genAll :: (String,String) -> String
genAll (sNumP, sK) = auxGen numP ""
   where
      auxGen 0 s = (genId numP) ++ s ++ genObj
      auxGen k s = auxGen (k-1) ((genP k 2) ++ s)
      -- k = read sK
      numP = read sNumP

-- Gets the two first elements of a list
fstTwo :: [a] -> (a,a)
fstTwo l = (head l, head (tail l))

-- Prints the automata generated for a number of processes given as an argument
main = fmap (genAll.fstTwo) getArgs >>= putStr
