-- Generation of timed automata modeling trains example from UPPAAL
--  Train 1 and 2 should not cross the gate together
import System.Environment

-- Generation of the kth train
genT :: Int -> String
genT k =
   "auto train" ++ (show k) ++ "\n" ++
   "{\n" ++
   "   safe -- appr" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> await\n" ++
   "   await -- ok" ++ (show k) ++ " -> appr\n" ++
   "   start -- toCross" ++ (show k) ++ " | x" ++ (show k) ++ "<=15 x" ++ (show k) ++ ">=7 x" ++ (show k) ++ ":=0  -> cross\n" ++
   "   cross -- leave" ++ (show k) ++ " | x" ++ (show k) ++ ">=3  x" ++ (show k) ++ "<=5 -> safe\n" ++
   "   appr  -- toCross" ++ (show k) ++ " | x" ++ (show k) ++ "<=20 x" ++ (show k) ++ ">=10 x" ++ (show k) ++ ":=0 -> cross\n" ++
   "   await -- stop" ++ (show k) ++ " | x" ++ (show k) ++ "<=10 -> stop\n" ++
   "   stop  -- go" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> start\n" ++
   "}\n"

-- Generation of gate with numT trains
-- aux deals with empty queues
-- aux2 deals with arrival in queues
-- aux3 deals with elements of queues
-- at least 3 trains should be considered
genGate :: Int -> String
genGate numT = 
   "auto gate\n" ++
   "{\n" ++
   (aux 1) ++
   "}\n"
   where
      aux t
         | t > numT = ""
         | otherwise  = 
               "   free0 -- appr" ++ (show t) ++ " | x:=0 -> ok" ++ (show t) ++ "\n" ++ 
               (aux (t + 1)) ++
               (aux2 1 [t]) ++
               (aux3 [t])
      aux2 t file
         | t > numT = ""
         | (any (t==) file) = (aux2 (t + 1) file)
         | otherwise =
               "   occ" ++ (pq file) ++ " -- appr" ++ (show t) ++ " | x:=0 -> occ" ++ (pq (file ++ [t])) ++ "s\n" ++
               (aux2 (t + 1) file) ++
               (aux2 1 (file ++ [t])) ++
               (aux3 (file ++ [t]))
      aux3 [] = ""
      aux3 [hd] =
         "   ok" ++ (show hd) ++ " -- ok" ++ (show hd) ++ " | x==0 -> occ" ++ (show hd) ++ "\n" ++
         "   free" ++ (show hd) ++ " -- go" ++ (show hd) ++ " -> occ" ++ (show hd) ++ "\n" ++
         "   occ" ++ (show hd) ++ " -- leave" ++ (show hd) ++ " -> free0\n"
      aux3 l
         | ((length l) == numT) =
               "   occ" ++ (pq l) ++ "s -- stop" ++ (show (last l)) ++ " | x == 0 -> occ" ++ (pq l) ++ "\n" ++
               "   occ" ++ (pq l) ++ " -- leave" ++ (show (head l)) ++ " -> free" ++ (pq (tail l)) ++ "\n"
         | otherwise =
               "   free" ++ (pq l) ++ " -- go" ++ (show (head l)) ++ " -> occ" ++ (pq l) ++ "\n" ++
               "   occ" ++ (pq l) ++ "s -- stop" ++ (show (last l)) ++ " | x == 0 -> occ" ++ (pq l) ++ "\n" ++
               "   occ" ++ (pq l) ++ " -- leave" ++ (show (head l)) ++ " -> free" ++ (pq (tail l)) ++ "\n"
      pq [] = ""
      pq (hd : tl) = (show hd) ++ (pq tl)


-- Generation of the objective
genObj :: String
genObj  = "objective train1.cross & train2.cross"

-- Generation of the automata for numP trains
genAll :: (String,String) -> String
genAll (sNumP, sK) = auxGen numP ""
   where
      auxGen 0 s = (genGate numP) ++ s ++ genObj
      auxGen k s = auxGen (k-1) ((genT k) ++ s)
      -- k = read sK
      numP = read sNumP

-- Gets the two first elements of a list
fstTwo :: [a] -> (a,a)
fstTwo l = (head l, head (tail l))

-- Prints the automata generated for a number of trains given as an argument
main = fmap (genAll.fstTwo) getArgs >>= putStr
