-- Generation of timed automata modeling trains example from UPPAAL
-- the queue of trains has been replaced by a set of trains
--  Train 1 and 2 should not cross the gate together
import System.Environment
import Data.List

-- Generation of the kth train
genT :: Int -> String
genT k =
   "auto train" ++ (show k) ++ "\n" ++
   "{\n" ++
   "   safe -- appr" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> await\n" ++
   "   await -- ok" ++ (show k) ++ " -> appr\n" ++
   "   start -- toCross" ++ (show k) ++ " | x" ++ (show k) ++ "<=15 x" ++ (show k) ++ ">=7 x" ++ (show k) ++ ":=0  -> cross\n" ++
   "   cross -- leave" ++ (show k) ++ " | x" ++ (show k) ++ ">=3  x" ++ (show k) ++ "<=5 -> safe\n" ++
   "   appr  -- toCross" ++ (show k) ++ " | x" ++ (show k) ++ "<=20 x" ++ (show k) ++ ">=10 x" ++ (show k) ++ ":=0 -> cross\n" ++
   "   await -- stop" ++ (show k) ++ " | x" ++ (show k) ++ "<=10 -> stop\n" ++
   "   stop  -- go" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> start\n" ++
   "}\n"

-- Generation of gate with numT trains
genGate :: Int -> String
genGate numT = 
   "auto gate\n" ++
   "{\n" ++
   (auxFree0 1) ++
   (auxOk 1) ++
   (auxFree allSets) ++
   (auxAppr allSets) ++
   "}\n"
   where
      auxFree0 k
         | k > numT = ""
         | otherwise = 
            "   free0 -- appr" ++ (show k) ++ " | x:=0 -> ok" ++ (show k) ++ "\n" ++
            (auxFree0 (k + 1))
      auxOk k
         | k > numT = ""
         | otherwise =
            "   ok" ++ (show k) ++ " -- ok" ++ (show k) ++ " | x==0 -> occ" ++ (show k) ++ "v" ++ (show k) ++ "\n" ++
            (auxOk (k + 1))
      auxFree [] = ""
      auxFree (h : t) =
         (auxauxFree h h) ++
         (auxFree t)
      auxauxFree _ [] = ""
      auxauxFree [] _ = ""
      auxauxFree (h : t) l =
         "   free" ++ (printSet l) ++ " -- go" ++ (show h) ++ " -> occ" ++ (show h) ++ "v" ++ (printSet l) ++ "\n" ++
         "   occ" ++ (show h) ++ "v" ++ (printSet l) ++ " -- leave" ++ (show h) ++ " -> free" ++ (printSet (delete h l)) ++ "\n" ++
         (auxauxFree t l)
      auxAppr [] = ""
      auxAppr (h : t) =
         (auxauxAppr h h 1) ++
         (auxAppr t)
      auxauxAppr _ [] _ = ""
      auxauxAppr [] _ _ = ""
      auxauxAppr (h : t) l k
         | k > numT = auxauxAppr t l 1
         | (any (k==) l) = auxauxAppr (h : t) l (k + 1)
         | otherwise =
            "   occ" ++ (show h) ++ "v" ++ (printSet l) ++ " -- appr" ++ (show k) ++ " | x:=0 -> occ" ++ (show h) ++ "v" ++ (printSet (insert k l)) ++ "v" ++ (show k) ++ "\n" ++
            "   occ" ++ (show h) ++ "v" ++ (printSet (insert k l)) ++ "v" ++ (show k) ++ " -- stop" ++ (show k) ++ " | x==0 -> occ" ++ (show h) ++ "v" ++ (printSet (insert k l)) ++ "\n" ++
            (auxauxAppr (h : t) l (k + 1))
      fullList k
         | k > numT = []
         | otherwise = k : (fullList (k + 1))
      allSets = subsequences (fullList 1)
      printSet [] = "0"
      printSet [k] = (show k)
      printSet (h : t) = (show h) ++ "x" ++  (printSet t)


-- Generation of the objective
genObj :: String
genObj  = "objective train1.cross & train2.cross"

-- Generation of the automata for numP trains
genAll :: (String,String) -> String
genAll (sNumP, sK) = auxGen numP ""
   where
      auxGen 0 s = (genGate numP) ++ s ++ genObj
      auxGen k s = auxGen (k-1) ((genT k) ++ s)
      -- k = read sK
      numP = read sNumP

-- Gets the two first elements of a list
fstTwo :: [a] -> (a,a)
fstTwo l = (head l, head (tail l))

-- Prints the automata generated for a number of trains given as an argument
main = fmap (genAll.fstTwo) getArgs >>= putStr
