-- Generation of timed automata modeling trains example of B. Berthomieu
--  cf. State class constructions for branching analysis of Time Petri nets
--  Train 1 should not cross while the gate is open
import System.Environment

-- Generation of the kth train
genT :: Int -> String
genT k =
   "auto train" ++ (show k) ++ "\n" ++
   "{\n" ++
   "   far -- appr" ++ (show k) ++ " | x" ++ (show k) ++ ":=0 -> close\n"  ++
   "   close -- in" ++ (show k) ++ " | x" ++ (show k) ++ "<=5 x" ++ (show k) ++ ">=3 x" ++ (show k) ++ ":=0 -> on\n"  ++
   "   on -- ex" ++ (show k) ++ " | x" ++ (show k) ++ "<=4 x" ++ (show k) ++ ">=2 x" ++ (show k) ++ ":=0 -> left\n" ++
   "   left -- exit" ++ (show k) ++ " | x" ++ (show k) ++ "==0 -> free\n" ++
   "   [close x" ++ (show k) ++ "<=5]\n" ++
   "   [on x" ++ (show k) ++ "<=4]\n" ++
   "   [left x" ++ (show k) ++ "<=0]\n" ++
   "}\n"

-- Generation of the tail of the controller when there are n trains
genC :: Int -> String
genC n =
   "auto controller\n" ++
   "{\n" ++
   (auxApprAllFar 1) ++
   "   coming -- down | x==0 -> contains1\n" ++
   (auxApprExit 2) ++
   (auxExitAllFar 1) ++
   "   leaving -- up | x==0 -> allFar\n" ++
   "   [coming x<=0]\n" ++
   "   [leaving x<=0]\n" ++ 
   "}\n"
   where
      auxApprAllFar nn
         | nn > n = ""
         | otherwise =
            "   allFar -- appr" ++ (show nn) ++ " | x:=0 -> coming\n" ++
            (auxApprAllFar (nn + 1))
      auxExitAllFar nn
         | nn > n = ""
         | otherwise =
            "   contains1 -- exit" ++ (show nn) ++ " | x:=0 -> leaving\n" ++
            (auxExitAllFar (nn + 1))
      auxApprExit nn
         | nn > n = ""
         | otherwise =
            (auxAppr nn 1) ++
            (auxApprExit (nn + 1)) ++
            (auxExit nn 1)
      auxAppr nn k
         | k > n = ""
         | otherwise =
            "   contains" ++ (show (nn - 1)) ++ " -- appr" ++ (show k) ++ " -> contains" ++ (show nn) ++ "\n" ++
            (auxAppr nn (k + 1))
      auxExit nn k
         | k > n = ""
         | otherwise =
            "   contains" ++ (show nn) ++ " -- exit" ++ (show k) ++ " -> contains" ++ (show (nn - 1)) ++ "\n" ++
            (auxExit nn (k + 1))

-- Generation of gate with numT trains
genGate :: Int -> String
genGate numT = 
   "auto gate\n" ++
   "{\n" ++
   "   open -- down | y:=0 -> startLowering\n" ++
   "   startLowering -- lowering | y<=2 y>=1 -> closed\n" ++
   "   closed -- up | y:=0 -> startRaising\n" ++
   "   startRaising -- down | y :=0 -> startLowering\n" ++
   "   startRaising -- raising | y<=2 y>=1 -> open\n" ++
   "   [startLowering y<=2]\n" ++
   "   [startRaising y<=2]\n" ++ 
   "}\n"

-- Generation of the objective
genObj :: String
genObj  = "objective train1.on & gate.open"

-- Generation of the automata for numP trains
genAll :: (String,String) -> String
genAll (sNumP, sK) = auxGen numP ""
   where
      auxGen 0 s = (genGate numP) ++ (genC numP) ++ s ++ genObj
      auxGen k s = auxGen (k-1) ((genT k) ++ s)
      -- k = read sK
      numP = read sNumP

-- Gets the two first elements of a list
fstTwo :: [a] -> (a,a)
fstTwo l = (head l, head (tail l))

-- Prints the automata generated for a number of trains given as an argument
main = fmap (genAll.fstTwo) getArgs >>= putStr
